;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Stylized Spatial-Social Subsystem based on Luhmann's theory
;;
;; Author: Marcos A. S. d. Silva   E-mail: marcos.santos-silva@embrapa.br
;;
;; Date: 30/05/2017        Version: 2.0.0
;;
;; OpenABM site: https://www.openabm.org/model/5543/version/3/view
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


globals [
  cells
  cellsEgo
  increments
  incrementsEgo

  aaverage
  moransI
  Z
  seed
  quit
  expected
  serial#
  #patches



  my_min-pxcor
  my_max-pxcor
  my_min-pycor
  my_max-pycor

  w-matrix
  board-matrix
  all-neighbors4
  Moran
  list-Moran


  AlterCapacity
  EgoCapacity

  AlterInfluence
  EgoInfluence

  list-Alter
  list-Ego
  ]


extensions [matrix]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Agent-S is represented by the set of patches
;; Each patch has
;;     stakes -> real values in [0,100]
;;     active -> can be active 1 or not 0
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
patches-own [stakes active]


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Agent-P -> turtles
;; There are two types of Agent-P
;;      Alter
;;      Ego
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
breed [alters alter]
breed [egos ego]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Each Agent-P has
;;     capacity -> real value used to change the patches' stakes
;;     memory -> a list of the level of authority (individual power)
;;     strategy -> 0 changes patches' stakes completely at random
;;                 1 changes patches' stakes in one small area of
;;                   the spatial board defined by the "ref-patch"
;;     activitiy ->
;;     positive-feedback ->
;;     rate-success ->
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
turtles-own [capacity memory strategy ref-patch activity positive-feedback rate-success]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Setup procedure
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to setup
  ca

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Resize patches according to the BoardSize
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set my_min-pxcor 0
  set my_max-pxcor BoardSize - 1
  set my_min-pycor 0
  set my_max-pycor BoardSize - 1
  resize-world my_min-pxcor my_max-pxcor my_min-pycor my_max-pycor

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Init
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set cells []
  set cellsEgo []


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Init patches' colors with
  ;; grey's scales
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ask patches [
    set active 1
    set pcolor scale-color black stakes 0 10
    set stakes random 100
  ]


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Color patches according to the stakes' values
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  color-patches

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Agent-S changes the accessibilities to the patches according to
  ;; their authorities
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  update-activity

  set list-Alter []
  set list-Ego []

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Count the number of patches under authority of Alters and Egos
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  count-influence


  set #patches count patches


  create-alters num-agents-Alter
  create-egos num-agents-Ego

  ask alters [set capacity init-capacity-Alter]
  ask egos [set capacity init-capacity-Ego]

  ask turtles [
    set memory []
    set strategy 0
    set activity matrix:make-constant BoardSize BoardSize 0
    set positive-feedback matrix:make-constant BoardSize BoardSize 0
    set rate-success matrix:make-constant BoardSize BoardSize 0
    ]


  set w-matrix create-w-matrix
  set all-neighbors4 get-all-neighbors4
  set Moran getMoran
  set list-Moran []
  set list-Moran fput Moran list-Moran

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Count the capacities of Alters and Egos at time t
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  count-capacities



  ask Alters [ set shape "circle" ]
  ask Alters [ set color red ]
  ask Alters [ set size 0.5 ]

  ask Egos [ set shape "circle" ]
  ask Egos [ set color blue ]
  ask Egos [ set size 0.5 ]

  reset-ticks
end

to-report sort-with [ key lst ]
  report sort-by [(runresult key ?1) < (runresult key ?2)] lst
end


;;sort-with task [ item 1 ? ] my-list

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Go procedure
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to go
  tick

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; If it is time to check if the Turtles should change their behaviors then
  ;; call choose-strategy
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  if ((ticks > time-analyze-behavior) and (ticks mod time-analyze-behavior = 0) and change-strategy?) [
     choose-strategy
  ]


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Move all Alters to another patch (if there is any active) called "cells"
  ;; according to the current strategy
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ask alters [
    ifelse (capacity > 0) [
       if any? patches with [active = 1] [
         ifelse (strategy = 0) [
           move-to one-of patches with [active = 1]
           set cells patch-here
           set increments capacity * ( random-float rate )

         ][ ifelse (strategy = 1) [
           let _center ref-patch
           if (_center != nobody) [
           let _candidates patches with [distance _center < radius-searching and active = 1]
           if (any? _candidates) [
              move-to one-of _candidates
              set cells patch-here
              set increments capacity * ( random-float rate )

           ]
           ]
         ]
         [
            ;; Strategy
            let _rate-success matrix:copy rate-success
            set _rate-success _rate-success matrix:* (2 * 0.5 - 1)
            set _rate-success _rate-success matrix:+ (1 - 0.5)
            ;; let _candidates patches with [(pxcor in ) and (pycor in ) ]
         ]
       ]
       ]
    ][die]
  ]

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Move all Egos to another patch (if there is any active=1) called "cellsEgo"
  ;; according to the current strategy
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ask egos [
    ifelse (capacity > 0) [
      if any? patches with [active = 1] [
        ifelse (strategy = 0) [
           move-to one-of patches with [active = 1]
           set cellsEgo patch-here
           set incrementsEgo capacity * ( random-float rate )
           ;;matrix:set activity (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
        ][
           let _center ref-patch
           if (_center != nobody) [
           let _candidates patches with [distance _center < radius-searching and active = 1]
           if (any? _candidates ) [
              move-to one-of _candidates
              set cellsEgo patch-here
              set incrementsEgo capacity * ( random-float rate )
              ;;matrix:set activity (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
           ]
           ]
        ]
      ]
    ][die]
  ]


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Upadate patches' stakes according to the Alters and Egos actions (increments and incrementsEgo)
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ask patch-set cells [ifelse (stakes - increments >= 0) [set stakes stakes - increments][set stakes 0]]
  ask patch-set cellsEgo [ifelse (stakes + incrementsEgo <= 100) [set stakes stakes + incrementsEgo][set stakes 100]]



  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Color patches according to the values of stakes
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  color-patches


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Agent-S changes the accessibilities to the patches according to
  ;; their authorities
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  update-activity

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Count the number of patches under authority of Alters and Egos
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  count-influence

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Calculates the Moran's I statistics at time t
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set Moran getMoran


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Store the Moran's I statistics at the t in the list list-Moran
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set list-Moran fput Moran list-Moran

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Count the capacities of Alters and Egos at time t
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  count-capacities
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Choose Turtles strategies according to the diff (delta-Ego and delta-Alter) of level of authority between
;; last tick and a time ago (delay-observation)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to choose-strategy
  if ((ticks > delay-observation) and (delay-observation > 0))[
    let delta-Alter (first list-Alter - item (delay-observation - 1) list-Alter )
    let delta-Ego (first list-Ego - item (delay-observation - 1) list-Ego )
    ifelse ( delta-Ego < 0 ) [
      ask Egos [
        set strategy 1
        set ref-patch one-of patches with [pcolor = blue and active = 1]
        matrix:set activity (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
        ]
    ][ask Egos [
      set strategy 0
      set ref-patch nobody
      matrix:set positive-feedback (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
      matrix:set activity (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
      matrix:set rate-success (BoardSize - 1 - ycor) xcor  (matrix:get positive-feedback (BoardSize - 1 - ycor) xcor) / (matrix:get activity (BoardSize - 1 - ycor) xcor)
      ]
    ]
    ifelse ( delta-Alter < 0 ) [
      ask Alters [
        set strategy 1
        set ref-patch one-of patches with [pcolor = red and active = 1]
        matrix:set activity (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
        ]
    ][ask Alters [
      set strategy 0
      set ref-patch nobody
      matrix:set positive-feedback (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
      matrix:set activity (BoardSize - 1 - ycor) xcor  (1 + matrix:get activity (BoardSize - 1 - ycor) xcor)
      matrix:set rate-success (BoardSize - 1 - ycor) xcor  (matrix:get positive-feedback (BoardSize - 1 - ycor) xcor) / (matrix:get activity (BoardSize - 1 - ycor) xcor)
      ]
    ]
  ]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Agent-S action
;;
;; Upadate the status of each patch
;; If a patche and its neighbors are unde the same authority then set active equal to zero (Inactive)
;; otherwise keep the status as active (active  = 1 )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to update-activity
  ask patches [if ((pcolor != gray) and ([pcolor] of neighbors4 = [red red red red] ) and (pcolor = red) )[set active 0 ask neighbors4 [set active 0]]  ]
  ask patches [if ((pcolor != gray) and ([pcolor] of neighbors4 = [blue blue blue blue] ) and (pcolor = blue) )[set active 0 ask neighbors4 [set active 0]]  ]
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Color patches according to two thresholds
;; If stake <= 30 then RED (patches under authority of Alters)
;; If stake >= 70 then BLUE (patches under authority of Egos)
;; otherwise GRAY
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to color-patches
ask patches [
    ifelse (stakes <= 30) [
         set pcolor red
      ]
    [
      ifelse (stakes >= 70) [set pcolor blue][set pcolor gray]
    ]
    ]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Count capacity of each type of turtles (Alter and Ego)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to count-capacities
  ask Egos [ set EgoCapacity sum [capacity] of Egos ]
  ask Alters [ set AlterCapacity sum [capacity] of Alters ]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Influence = authority over a patch
;; Count how many patches is under the authority of Alter (AlterInfluence)
;;
;; Count how many patches is under the authority of Ego (EgoInfluence)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to count-influence
  ask patches [ set AlterInfluence count patches with [pcolor = red]]
  set list-Alter fput AlterInfluence list-Alter
  ask patches [ set EgoInfluence count patches with [pcolor = blue] ]
  set list-Ego fput AlterInfluence list-Ego
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;
;; Moran's I statistics calculation
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Create a matrix Anxn from the patches, n = BoardSize
;; Each element aij = stake of the patch in the position i j
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report create-board-matrix
  set board-matrix matrix:make-constant BoardSize BoardSize 0
  let list-patches sort patches
  let i 0
  let j 0
  let k 0
  while [i < BoardSize][
    set j 0
    while [ j < BoardSize ] [
      matrix:set board-matrix i j [stakes] of patch ([pxcor] of item k sort patches) ([pycor] of item k sort patches)
      set k k + 1
      set j j + 1
    ]
    set i i + 1
  ]
  report board-matrix
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Used to support the calculation of the neighbor matrix (rook-neighbor)
;; returns the coordinates i j of a patch in the position pos
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report get-coord [ pos ]
  let list-coord []
  let B BoardSize
  let L floor ( pos  / B )


  let C 0
  ifelse ( pos < B ) [
    set C pos
  ][ ifelse ( pos mod B = 0) [
    set C B
    set L L - 1
    ][
    set C pos mod B
    ]
  ]
  set list-coord lput L list-coord
  set list-coord lput (C - 1) list-coord
  report list-coord
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Used to support the calculation of the neighbor matrix (rook-neighbor)
;; returns the position pos of a patch positioned at the
;; coordinates i j
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report get-pos [ coord ]
  let pos-tmp (item 0 coord) * (BoardSize) + (item 1 coord + 1)
  report (pos-tmp - 1)
end

to-report get-weightedW [ w ]
  report 0
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Create the neighborhood matrix Wmxm, m = BoardSizexBoardSize
;; Each element
;;     wij = 1, if the patch at position i is neighbor of the patch in the position j
;;     wij = 0, otherwise
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report create-w-matrix
  let M #patches
  let W matrix:make-constant M M 0
  let i 0
  let tmp-coord []
  let N1 0
  let N2 0
  let N3 0
  let N4 0
  let list-coord []

  while [i < M] [
    set N1 -1
    set N2 -1
    set N3 -1
    set N4 -1


    set tmp-coord get-coord (i + 1)


    if (item 0 tmp-coord - 1 >= 0) [
      set list-coord []
      set list-coord lput (item 0 tmp-coord - 1) list-coord
      set list-coord lput (item 1 tmp-coord) list-coord
      set N1 get-pos list-coord
      ]


    if (item 1 tmp-coord + 1 < BoardSize ) [
      set list-coord []
      set list-coord lput (item 0 tmp-coord) list-coord
      set list-coord lput (item 1 tmp-coord + 1) list-coord
      set N2 get-pos list-coord


      ]

    if (item 0 tmp-coord + 1 < BoardSize) [
      set list-coord []
      set list-coord lput (item 0 tmp-coord + 1) list-coord
      set list-coord lput (item 1 tmp-coord) list-coord
      set N3 get-pos list-coord
      ]

    if (item 1 tmp-coord - 1 >= 0) [
      set list-coord []
      set list-coord lput (item 0 tmp-coord) list-coord
      set list-coord lput (item 1 tmp-coord - 1) list-coord
      set N4 get-pos list-coord
    ]

    if (N1 >= 0) [matrix:set W i N1 1]
    if (N2 >= 0) [matrix:set W i N2 1]
    if (N3 >= 0) [matrix:set W i N3 1]
    if (N4 >= 0) [matrix:set W i N4 1]



    set i i + 1
  ]

  report W
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Computes the avegare of all stakes. Each patche is associated
;; with one stake value
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report calculate-average

  let atotal 0
  ask patches [
    set atotal ( atotal + stakes )
  ]
  set aaverage ( atotal / #patches )
  report aaverage
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Returns all four neighbors (rook-type neighborhood) of a patch
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report get-all-neighbors4
  let sumup 0
  let i 0
  let j 0
  let k 0
  while [i < #patches ][
    set j 0
    while [ j < #patches ] [
      set sumup sumup + matrix:get w-matrix i j
      set j j + 1
    ]
    set i i + 1
  ]
  report sumup
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Calculates the denominator of the Moran's I statistics
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report calculate-denominator [ D_mean ]

  let denominator 0
  ask patches [
    set denominator (denominator + ( stakes - D_mean ) * ( stakes - D_mean ) )
  ]
  report denominator
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Calculates the Moran's I statistics
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report getMoran

  let D create-board-matrix
  set D matrix:transpose D
  let D_mean calculate-average
  let Denominator calculate-denominator D_mean

  let WW create-w-matrix


  let coordi []
  let coordj []
  let Numerator 0
  let tmp 0

   let i 0
   while [ i < #patches ] [
    let j 0
    while [ j < #patches ] [
      if ((matrix:get WW i j) = 1) [
          set coordi get-coord (i + 1)
          set coordj get-coord (j + 1)
          set tmp ( matrix:get WW i j ) * ( matrix:get D item 0 coordi item 1 coordi  - D_mean ) * ( matrix:get D item 0 coordj item 1 coordj - D_mean )
          set Numerator Numerator + tmp
      ]
      set j j + 1
    ]
    set i i + 1
  ]


  ifelse (Denominator = 0 ) [
    report 1
    stop
  ][
  report ( #patches / all-neighbors4 ) * ( Numerator / Denominator )
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
392
24
864
517
-1
-1
25.7
1
10
1
1
1
0
1
1
1
0
17
0
17
0
0
1
ticks
30.0

BUTTON
21
17
84
50
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
92
17
156
52
NIL
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
167
18
230
51
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
25
235
197
268
num-agents-Alter
num-agents-Alter
1
50
1
1
1
NIL
HORIZONTAL

SLIDER
25
368
197
401
num-agents-Ego
num-agents-Ego
1
50
1
1
1
NIL
HORIZONTAL

PLOT
880
24
1080
174
Moran's I
NIL
NIL
0.0
10.0
0.0
0.01
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot Moran"

PLOT
880
189
1080
339
Capacity of action
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "plot AlterCapacity"
"pen-1" 1.0 0 -13791810 true "" "plot EgoCapacity"

PLOT
881
354
1081
504
Authority
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "plot AlterInfluence"
"pen-1" 1.0 0 -13791810 true "" "plot EgoInfluence"

SLIDER
25
270
197
303
init-capacity-Alter
init-capacity-Alter
10
1000
400
10
1
NIL
HORIZONTAL

SLIDER
26
404
198
437
init-capacity-Ego
init-capacity-Ego
10
1000
400
10
1
NIL
HORIZONTAL

TEXTBOX
26
204
176
222
Alter
11
0.0
1

TEXTBOX
25
343
175
361
Ego
11
0.0
1

MONITOR
29
457
93
502
Moran's I
precision Moran 5
17
1
11

SLIDER
26
127
198
160
BoardSize
BoardSize
2
18
18
1
1
NIL
HORIZONTAL

PLOT
1095
24
1295
174
Total energy of the spatial board
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot aaverage"

INPUTBOX
141
62
191
122
rate
0.025
1
0
Number

SLIDER
217
150
389
183
delay-observation
delay-observation
0
100
100
10
1
NIL
HORIZONTAL

SLIDER
217
190
389
223
radius-searching
radius-searching
1
5
3
1
1
NIL
HORIZONTAL

SLIDER
217
112
390
145
time-analyze-behavior
time-analyze-behavior
0
1000
500
1
1
NIL
HORIZONTAL

SWITCH
218
70
390
103
change-strategy?
change-strategy?
0
1
-1000

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.3.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="ResultsMABS2017" repetitions="10" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="20000"/>
    <metric>Moran</metric>
    <metric>AlterInfluence</metric>
    <metric>EgoInfluence</metric>
    <enumeratedValueSet variable="BoardSize">
      <value value="18"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-capacity-Ego">
      <value value="400"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="rate">
      <value value="0.025"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-analyze-behavior">
      <value value="500"/>
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents-Alter">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="AlterMemory">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="delay-observation">
      <value value="50"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="radius-searching">
      <value value="3"/>
      <value value="4"/>
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="EgoMemory">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="MaxMemory">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents-Ego">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="change-strategy?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-capacity-Alter">
      <value value="400"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment" repetitions="10" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="45000"/>
    <metric>Moran</metric>
    <metric>AlterInfluence</metric>
    <metric>EgoInfluence</metric>
    <enumeratedValueSet variable="BoardSize">
      <value value="18"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="MaxMemory">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-capacity-Alter">
      <value value="400"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="change-strategy?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents-Alter">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="EgoMemory">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-analyze-behavior">
      <value value="500"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="rate">
      <value value="0.025"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="delay-observation">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="radius-searching">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-capacity-Ego">
      <value value="400"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents-Ego">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="AlterMemory">
      <value value="true"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="EXP08" repetitions="10" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="20000"/>
    <metric>Moran</metric>
    <metric>AlterInfluence</metric>
    <metric>EgoInfluence</metric>
    <enumeratedValueSet variable="radius-searching">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="change-strategy?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-analyze-behavior">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-capacity-Alter">
      <value value="400"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="MaxMemory">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="delay-observation">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="EgoMemory">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents-Alter">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BoardSize">
      <value value="18"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="AlterMemory">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents-Ego">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-capacity-Ego">
      <value value="400"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="rate">
      <value value="0.025"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
